#!/usr/bin/perl
#
# A simple Perl program to generate an SMS formatted data file containing a
# ringtone sound for Nokia phones (and perhaps other phones that use the
# SMS standard).
#
# At present this program does absolutely nothing of much value other than
# demonstrate a method of assembling a working SMS RTPL packet to deliver
# ringtone data to compatible phone. This method should work with most
# Nokia phones that accept ringtones. If it works (or doesn't) with other
# types of phones, let me know.
#

#==========================================================================
# Copyright (c) 1999, 2000 by R. Steven Rainwater <steve@ncc.com>
# 
# http://www.ncc.com/freeware/
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#==========================================================================

# Notes: A WDP (Wireless Datagram Protocol) header for a ringtone message
# is hexadecimal data in text format and looks like this:
#
# //SCKL15810000010101 024A3A6D91BC8191BC819194819194041418B91BE92AC5A4AB16938C5A4E316410
#
# //SCKL	The WDP datagram identifier
# 1581		The RFC ringtone listener port number
# 1581		Origination port (?)
# 01		Reference number (?)
# 01		Total number of data segments
# 01		Sequence number of this segment
# 		Space (required)
# ringdata	Ringtone bit string, divided into bytes
#
# Some examples containing only one segment omit the origin port, ref number,
# and segmentation data so the origin and ref may be needed only to help
# re-assemble multi-segment SMS messages.
#
# The ringtone data is a bit string:
#
# 00000010	Command Length (usually a binary two in examples)
# 01001010	Command type: ringing tone programming
# 0011101	Command type: sound data
# 001		Sound data type: Basic Song
# 0100		Title length (1-16 ISO-8859-1characters)
# [title]
# 00000001	Number of sound sequences
# 000		Pattern Header
# 00		Pattern ID
# 1111		Loop value: Number of repititions (1111 = infinite, 0000 = once)
# Sound data bits follow (varies with each sound

# Legal Sound commands
#
# 000	Pattern header
# 001	Note [note command, duration command, extended-duration command]
# 010	Scale
# 011	Style
# 100	BPM
# 101	Volume

# Legal Note commands are:
#
# 0000	pause
# 0001	C
# 0010	C#
# 0011	D
# 0100	D#
# 0101	E
# 0110	F
# 0111	F#
# 1000	G
# 1001	G#
# 1010	A
# 1011	A#
# 1100	B

# Legal Note duration commands are:
#
# 000	Whole note
# 001	1/2 note
# 010	1/4 note
# 011	1/8 note
# 100	1/16 note
# 101	1/32 note

# Legal Note extended duration commands are:
# 
# 00	standard duration
# 01	dot
# 10	double dot
# 11	2/3 normal duration

# Legal Scale commands are:
#
# 00	A = 440hz (default)
# 01	A = 880hz
# 10	A = 1.76hz
# 11	A = 3.52hz

# Legal Style values are:
# 00	short pause between notes
# 01	no pause between notes
# 10	long pause between shortened notes

# Legal BPM commands are:
#
# 00000	 25
# 00001	 28
# 00010	 31
# 00011	 35
# 00100  40
# 00101	 45
# 00110	 50
# 00111	 56
# 01000	 63
# 01001	 70
# 01010	 80
# 01011	 90
# 01100	 100
# 01101	 112
# 01110	 125
# 01111	 140
# 10000	 160
# 10001	 180
# 10010	 200
# 10011	 225
# 10100	 250
# 10101	 285
# 10110	 320
# 10111	 355
# 11000	 400
# 11001	 450
# 11010	 500
# 11011	 565
# 11100	 635
# 11101	 715
# 11110	 800
# 11111  900

my $version = '0.1';

my $title = 'tst3';	# Title string < 16 chars max
my $bpm   = '10010000';	# 100 11111 (bpm command + 160 bpm value)
my $style = '01101';	# 011 01 (style command + style value)


my $tmp = '';
my $sounddata = '';
my $ringdata = '00000010010010100011101001';	# command length & type data

# Build the ringtone title byte count bit string
$tmp = unpack 'B*', pack 'C', (length $title);
$tmp=~s/^0000//;
$ringdata .= $tmp;

# Build the ringtone title bit string
foreach (split//, $title) {
    $ringdata .= (unpack 'B*', pack 'a1', $_);
}

# Hardcode for 1 sound sequence, 1 pattern w pattern ID
$ringdata .= '0000000100000';

# Hardcode for infinite number of play loops
$ringdata .= '1111';

# Build a simple hard-coded sound
# cmd 1
$sounddata = $style;

# cmd 2
$sounddata .= $bpm;	# add a bpm command

# cmd 3
$sounddata .= '01000';	# 010 00 (scale command + scale A=440hz)

# cmd 4
$sounddata .= '001';	# new note
$sounddata .= '0001';	# note=C
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 5
$sounddata .= '001';	# new note
$sounddata .= '0010';	# note=C#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 6
$sounddata .= '001';	# new note
$sounddata .= '0011';	# note=D
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 7
$sounddata .= '001';	# new note
$sounddata .= '0100';	# note=D#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 8
$sounddata .= '001';	# new note
$sounddata .= '0101';	# note=E
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 9
$sounddata .= '001';	# new note
$sounddata .= '0110';	# note=F
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 10
$sounddata .= '001';	# new note
$sounddata .= '0111';	# note=F#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 11
$sounddata .= '001';	# new note
$sounddata .= '1000';	# note=G
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 12
$sounddata .= '001';	# new note
$sounddata .= '1001';	# note=G#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 13
$sounddata .= '001';	# new note
$sounddata .= '1010';	# note=A
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 14
$sounddata .= '001';	# new note
$sounddata .= '1011';	# note=A#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 15
$sounddata .= '001';	# new note
$sounddata .= '1100';	# note=B
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 16
$sounddata .= '01001';	# 010 01 (scale command + scale A=880hz)

# cmd 17
$sounddata .= '001';	# new note
$sounddata .= '0001';	# note=C
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 18
$sounddata .= '001';	# new note
$sounddata .= '0010';	# note=C#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 19
$sounddata .= '001';	# new note
$sounddata .= '0011';	# note=D
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 20
$sounddata .= '001';	# new note
$sounddata .= '0100';	# note=D#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 21
$sounddata .= '001';	# new note
$sounddata .= '0101';	# note=E
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 22
$sounddata .= '001';	# new note
$sounddata .= '0110';	# note=F
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 23
$sounddata .= '001';	# new note
$sounddata .= '0111';	# note=F#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 24
$sounddata .= '001';	# new note
$sounddata .= '1000';	# note=G
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 25
$sounddata .= '001';	# new note
$sounddata .= '1001';	# note=G#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 26
$sounddata .= '001';	# new note
$sounddata .= '1010';	# note=A
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 27
$sounddata .= '001';	# new note
$sounddata .= '1011';	# note=A#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 28
$sounddata .= '001';	# new note
$sounddata .= '1100';	# note=B
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 29
$sounddata .= '01010';	# 010 10 (scale command + scale A=1.76khz)

# cmd 30
$sounddata .= '001';	# new note
$sounddata .= '0001';	# note=C
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 31
$sounddata .= '001';	# new note
$sounddata .= '0010';	# note=C#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 32
$sounddata .= '001';	# new note
$sounddata .= '0011';	# note=D
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 33
$sounddata .= '001';	# new note
$sounddata .= '0100';	# note=D#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 34
$sounddata .= '001';	# new note
$sounddata .= '0101';	# note=E
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 35
$sounddata .= '001';	# new note
$sounddata .= '0110';	# note=F
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 36
$sounddata .= '001';	# new note
$sounddata .= '0111';	# note=F#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 37
$sounddata .= '001';	# new note
$sounddata .= '1000';	# note=G
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 38
$sounddata .= '001';	# new note
$sounddata .= '1001';	# note=G#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 39
$sounddata .= '001';	# new note
$sounddata .= '1010';	# note=A
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 40
$sounddata .= '001';	# new note
$sounddata .= '1011';	# note=A#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 41
$sounddata .= '001';	# new note
$sounddata .= '1100';	# note=B
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 42
$sounddata .= '01011';	# 010 11 (scale command + scale A=3.52khz)

# cmd 43
$sounddata .= '001';	# new note
$sounddata .= '0001';	# note=C
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 44
$sounddata .= '001';	# new note
$sounddata .= '0010';	# note=C#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 45
$sounddata .= '001';	# new note
$sounddata .= '0011';	# note=D
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 46
$sounddata .= '001';	# new note
$sounddata .= '0100';	# note=D#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 47
$sounddata .= '001';	# new note
$sounddata .= '0101';	# note=E
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 48
$sounddata .= '001';	# new note
$sounddata .= '0110';	# note=F
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 49
$sounddata .= '001';	# new note
$sounddata .= '0111';	# note=F#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 50
$sounddata .= '001';	# new note
$sounddata .= '1000';	# note=G
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 51
$sounddata .= '001';	# new note
$sounddata .= '1001';	# note=G#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 52
$sounddata .= '001';	# new note
$sounddata .= '1010';	# note=A
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 53
$sounddata .= '001';	# new note
$sounddata .= '1011';	# note=A#
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 54
$sounddata .= '001';	# new note
$sounddata .= '1100';	# note=B
$sounddata .= '10100';	# 1/32 note, standard duration

# cmd 55
$sounddata .= '001';	# new note
$sounddata .= '0000';	# note= pause
$sounddata .= '00000';	# whole note, standard duration


# add sound to ringdata
# number of commands in pattern
$ringdata .= unpack 'B*', pack 'C', 55;

# add the sounddata bit string
$ringdata .= $sounddata;

# padd to an 8 bit boundy so we can convert to hex
print length $ringdata, "\n";
my $i = ((length $ringdata) % 8);
if ($i > 0) { foreach (1 .. $i) { $ringdata .= '0'; } }
print length $ringdata, "\n";

# add the end command
$ringdata .= '00000000';

print "Final ring data bitstring:\n",$ringdata, "\n\n";

unless($ARGV[0]) { die "\n\ntonegen $version\nusage: tonegen.pl [smsfile]\n\n"; }
 
if (open SMSF, ">$ARGV[0]") {
    print SMSF '//SCKL15811581010101 ', uc (unpack 'H*', pack 'B*', $ringdata);
    print SMSF "\n";
    close SMSF;
}
